// Copyright (C) 2000 Dominic Letourneau (doumdi@yahoo.com)
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this file.  If not, write to the Free Software Foundation,
// 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef _PPMWRITER_CC_
#define _PPMWRITER_CC_

#include "PPMWriter.h"
#include "Object.h"
#include "ObjectRef.h"
#include "Exception.h"


NODE_INFO(PPMWriter,"Image", "FILENAME:IMAGE_IN", "IMAGE_OUT", "")

PPMWriter::PPMWriter(string nodeName, ParameterSet params)
 
   : Node(nodeName, params) {

  m_filenameID = addInput("FILENAME");
  m_imageID = addInput("IMAGE_IN");

  addOutput("IMAGE_OUT");
}

ObjectRef PPMWriter::getOutput (int output_id, int count) {
   
   int i;


   if (!hasOutput(output_id)) throw new NodeException (this, "Cannot getOutput id",__FILE__,__LINE__);

   if (count != processCount) {
      //We are updating our output only if needed
      

     try {
       
       //getting all data from our inputs.
       int OutputID = inputs[m_filenameID].outputID;
       String fname = object_cast<String> (inputs[m_filenameID].node->getOutput(OutputID,count));
     
       OutputID = inputs[m_imageID].outputID;
       ObjectRef image =inputs[m_imageID].node->getOutput(OutputID,count);


       Image *im = dynamic_cast<Image*>(image.get());
       if (!im){
	 throw new NodeException (this, "Cannot get image",__FILE__,__LINE__);
       }
       
       //reading fname
       im->write_ppm(fname);
       
       if (!im->get_data()) {
	 throw new NodeException (this, "Invalid ppm file",__FILE__,__LINE__);
       }
       
       output = image;
       
       
     } //end of try block
     catch (BaseException *e) {
       //Something weird happened
       //e->print();
       throw e->add(new NodeException (this,string("error caught in PPMWriter::getOutput(int,int)") + 
				       inputs[m_filenameID].node->getName()
				       , __FILE__,__LINE__));
     }      
     
      
      
     //updating processCount
     processCount = count;                 
      
      
   }
   
   return output;
 
}
#endif
